import std/[json, logging, options]
import docopt

import wdiwtlt/private/cliconstants
import wdiwtlt/[cli, config, db]

when isMainModule:

 var logFile: File
 try:
  let args = docopt(USAGE, version = WDIWTLT_VERSION)

  let consoleLogger = newConsoleLogger(
    levelThreshold =
      if args["--verbose"]: lvlDebug
      else: lvlInfo,
    fmtStr="pit - $levelname: ")
  logging.addHandler(consoleLogger)

  let cfg = loadConfig(args)

  if cfg.logFile.isSome:
    info "Logging to '" & cfg.logFile.get & "'"
    try: logFile = open(cfg.logFile.get)
    except:
      warn "Unable to open the log file '" & cfg.logFile.get & "'"
    let fileLogger = newFileLogger(logFile, lvlDebug)
    logging.addHandler(fileLogger)

  if args["init-library"]:
    info "Initializing a new library database at '" & cfg.dbPath & "'"
    let newDb = initDb(cfg.dbPath)
    newDb.persist

    info "Writing configuration to '" & cfg.cfgPath & "'"
    cfg.cfgPath.writeFile(pretty(%*{
      "dbPath": cfg.dbPath,
      "libraryPath": cfg.libraryPath
    }))

    quit(0)

  startCli(cfg)

 except:
  fatal getCurrentExceptionMsg()
  debug getStackTrace(getCurrentException())
  quit(QuitFailure)

 finally:
  if not logFile.isNil: logFile.close()
