import std/strformat

const WDIWTLT_VERSION* = "1.0.0"

const USAGE* = "wdiwtlt v" & WDIWTLT_VERSION & """

Usage:
    wdiwtlt [options]
    wdiwtlt --version
    wdiwtlt --help
    wdiwtlt init-library [options]

Options:
    -c --config <config-file>

        Config file for wdiwtlt CLI.

    -L --library-path <root-dir>

        The path to a local media library directory.

    -D --db-path <database-path>

        The path to the JSON database file.

    --log-file <log-file>

        Write debug logs to <log-file>

    --verbose

      Enable verbose debug logging to stdout.
"""

const FMT_MARK* = "∙"

const FC* = FMT_MARK & "COMMAND" & FMT_MARK
const FE* = FMT_MARK & "ERROR" & FMT_MARK
const FN* = FMT_MARK & "NORMAL" & FMT_MARK

const HELP_GENERAL* = fmt("""Available Commands:

    scan        Scan the media library for new files.
    rescan      Re-scan the media library, new and existing files.
    quit        Quit this program.
                Aliases: {FC}exit{FN}

A new user is advised to read the help section for the {FC}select{FN} command.

For a quick list of options, consider reading the {FC}summary{FN} section.

""")

const HELP_QUIT* = fmt("""
{FC}quit{FN}    Close the wdiwtlt program.
{FC}exit{FN}

""")

const HELP_SCAN* = fmt("""
{FC}scan{FN}

    Scan the media library for new files. This will skip over files that the
    library is already aware of.

""")

const HELP_RESCAN* = fmt("""
{FC}rescan{FN}

    Scan the media library, processing all files in the library directory,
    including files we have already scanned previously.

""")

const HELP_LIST* = fmt("""
{FC}list bookmarks{FN}              List all bookmarks.
{FC}list selection{FN}              List the currently selected items.
{FC}list <select-criteria>{FN}      Make a selection using the {FC}select{FN} syntax and then
                            list the selection.

""")

const HELP_SELECT* = fmt("""
{FC}select playing {{ album | artist | file | playlist | tag }}{FN}

    Select the currently playing items into the selection buffer. Specifically,
    this selects items that are associated with the currently playing media
    file.

{FC}select queued {{ albums | artists | files | playlists | tags }}{FN}

    Select the items currently in the queue.

{FC}select <count> random {{ albums | artists | files | playlists | tags }}{FN}

    Select one or more items randomly.

{FC}select <count> random {{ albums | artists | files | playlists | tags }} from <select-criteria>{FN}

    Make a selection, then select one or more items randomly from it.

{FC}select selected {{ album | artist | file | playlist | tag }}{FN}

    Select the items associated with the current selection buffer into the
    selection buffer. This is useful to change the type of the selection. For
    example, the following commands would select all of the albums in the
    library by Bob Dylan:

        select artist Bob Dylan
        select selected albums

    Another example, to select media files based on a set of tags:

        select tags instrumental orchestral
        select selected files

    This example would select all files that are tagged as *either*
    instrumental or orchestral

{FC}select absent files{FN}

    Select al media files which have entries in the database but are not
    actually present locally on disk.

{FC}select files tagged as <tag-name>...{FN}

    Select all media files tagged with the given tags. If multiple tags are
    given then only files which have all the given tags are selected. In
    contrast to the previous example:

        select files tagged as instrumental orchestral

    This selects all files that are tagged as *both* instrumental and
    orchestral.

{FC}select untagged files{FN}

    Select all media files that do not have any tags associated with them.


{FC}select {{ album | artist | file | playlist | tag }} where <criteria>...{FN}

    {FE}Not yet implemented.{FN}

{FC}select {{ album | artist | file | playlist | tag }} <id | name>{FN}

    Select a single item by ID or by name. When selecting by name, the name can
    include spaces and can be a substring of the whole name ("Lonely Hearts"
    for "Sgt. Pepper's Lonely Hearts Club Band" for example, quotations not
    required).

{FC}select {{ albums | artists | files | playlists | tags }} <id>...{FN}

    Select multiple items by ID. Multiple IDs can be given, separated by
    spaces. If no IDs are given, all of the items are returned.

{FC}queue{FN}

    Select the current play queue

""")

const HELP_CREATE* = fmt("""
{FC}create bookmark named <name>{FN}

    Create a new bookmark at the current play position in the currently playing
    playlist.

{FC}create bookmark named <name> on playlist <playlist-name | id> at <file-name | id>{FN}

    Create a bookmark on the named media file in the named playlist.

{FC}create playlist named <name>{FN}

    Create a new playlist.

{FC}create playlist named <name> from {{ queue | selection}}{FN}

    Create a new playlist and populate it with the contents of either the
    current play queue or the current selection.
""")

const HELP_DELETE* = fmt("""
{FC}delete playlist <name | id>
delete bookmark <name | id>{FN}

    Delete a playlist or bookmark.
""")

const HELP_PLAY* = fmt("""
{FC}play{FN}

    With no options, play the current file (inverse of pause).

{FC}play selection{FN}

    Clear the play queue, enqueue the current selection, and begin playback.

{FC}play bookmark <id | name>{FN}

    Load the bookmarked playlist as the play queue and begin playback at the
    bookmarked media file.

{FC}play <select-criteria>{FN}

    Make a selection using the {FC}select{FN} syntax and then play the selection.

""")

const HELP_ENQUEUE* = fmt("""
{FC}enqueue selection{FN}

    Add the media files for the selected items to the end of the current play
    queue.

{FC}enqueue <select-criteria>{FN}

    Make a selection using the {FC}select{FN} syntax and then enqueue
    the selection.

""")

const HELP_ADD* = fmt("""
{FC}add selection to playlist <id | name>{FN}

    Lookup a playlist by id or name (including partial match) then add the
    media files for the selected items to the end of that playlist.

{FC}add <select-criteria> to playlist <id | name>{FN}

    Lookup a playlist by id or name (including partial match), select a set of
    media files using the {FC}select{FN} syntax, then add the media files
    for the selected items to the end of that playlist.

""")

const HELP_REMOVE* = fmt("""
{FC}remove selection from {{ queue | selection }}{FN}
{FC}remove selection from playlist <id | name>{FN}

    Remove the media files for the current selection from the current
    selection, the current play queue, or from a playlist looked up by ID or
    name (including partial match).

{FC}remove <select-criteria> from {{ queue | selection }}{FN}
{FC}remove <select-criteria> from playlist <id | name>{FN}

    Make a selection using the {FC}select{FN} syntax then remove those media
    files from either the current selection, the play queue, or from a
    playlist looked up by ID or name (including partial match).

""")

const HELP_RANDOMIZE* = fmt("""
{FC}randomize queue{FN}
{FC}randomize playlist <id | name>{FN}

    Randomize the order of all elements in either the current play queue or the
    named playlist.

""")

const HELP_REORDER* = fmt("""
{FC}reorder queue move <file id | name> to <position>{FN}
{FC}reorder playlist <id | name> move <file id | name> to <position>{FN}

    Move a single file from  its current position to the named position in the
    play queue/playlist.

{FC}reorder queue as <file id>... starting at <position>{FN}
{FC}reorder playlist <id | name> as <file id>... starting at <position>{FN}

    Take a set of files in the playlist, reorder them into given order, and
    move them to the given starting position (defaults to the end of the
    playlist)

""")

const HELP_TAG* = fmt("""
{FC}tag <tag>...{FN}

    Tag the currently playing file with the given tags. Multiple tags may be
    provided, separated by spaces (tags cannot include spaces).

{FC}tag selection as <tag>...{FN}

    Tag all of the media files in the current selection with the given tags.
    Multiple tags may be provided separated by spaces (tags cannot include
    spaces).

{FC}tag <select-criteria> as <tag>...{FN}

    Make a selection using the {FC}select{FN} syntax then tag all of the
    media files in the selection with the given tags. Multiple tags may be
    provided separated by spaces (tags cannot include spaces).

""")
const HELP_UNTAG* = fmt("""
{FC}untag <tag>...{FN}

    Remove the given tags from the currently playing file. Multiple tags may be
    provided, separated by spaces (tags cannot include spaces).

{FC}untag selection as <tag>...{FN}

    Remove the given tags from all of the media files in the current selection.
    Multiple tags may be provided separated by spaces (tags cannot include
    spaces).

{FC}untag <select-criteria> as <tag>...{FN}

    Make a selection using the {FC}select{FN} syntax then remove the
    given tags from all of the media files in the selection. Multiple tags may
    be provided separated by spaces (tags cannot include spaces).

""")

const HELP_CLEAR* = fmt("""
{FC}clear{FN}                           Clear the terminal display.
{FC}clear queue{FN}                     Clear the play queue.
{FC}clear selection{FN}                 Clear the selection buffer.
{FC}clear playlist <id | name>{FN}      Clear the given playlist

""")
const HELP_PAUSE* = "{FC}pause{FN}     Pause playback."
const HALPE_STOP* = "{FC}stop{FN}      Stop playback."

const HELP_NEXT* = fmt(
"""
{FC}next <count>{FN}        Move forward in the play queue by <count> items. <count> is
                    optional and defaults to 1

""")

const HELP_PREV* = fmt(
"""
prev <count>        Move backward in the play queue by <count> items. <count> is
                    optional and defaults to 1

""")

const HELP_JUMP* = fmt("""
{FC}jump to <media file ID or name>{FN}

    Find the given media file by ID or name in the current play queue and
    resume playback starting from that file.

""")

const HELP_FF* = fmt("""
{FC}ff <amount> <unit>{FN}

    Jump forward in the playback of the current media by <amount> specified in
    <unit>s. <amount> must be an integer. <unit> may be one of: 'millisecons',
    'seconds', or 'minutes'. The following abbreviations are allowed: 'ms',
    'millis', 's', 'sec', 'm', 'min'.

""")

const HELP_RW* = fmt("""
{FC}rw <amount> <unit>{FN}

    Jump backward in the playback of the current media by <amount> specified in
    <unit>s. <amount> must be an integer. <unit> may be one of: 'millisecons',
    'seconds', or 'minutes'. The following abbreviations are allowed: 'ms',
    'millis', 's', 'sec', 'm', 'min'.

""")

const HELP_REPEAT* = fmt("""
{FC}repeat {{ all | one | none }}{FN}

    Set the playlist repeat mode to:
        all: repeat the entire play queue
        one: loop the currently playing song
        none: do not repeat.
""")

const HELP_VOL* = fmt(
"""
volume              Display the current volume setting.
volume <percent>    Set the volume. <percent> may be any value from 0 to 200.

""")

const HELP_HELP* = fmt(
"""
help summary        Display a quick summary of available commands.
help <commmand>     Display detailed information about the given command.""")


const HELP_SUMMARY* = fmt(
"""
Selecting files:

    select {{ album | artist | file | playlist | tag }} <id | name>
    select {{ albums | artists | files | playlists | tags }} where <criteria>
    select files tagged as <tag>... and not as <tag>...
    select playing {{ albums | artists | files | playlists | tags }}
    select queued {{ albums | artists | files | playlists | tags }}
    select absent files

    list selection
    list <selection-criteria>
    list selected {{ albums | artists | files | playlists | tags }}

Play and Controlling Media:

    play
    play selection
    play bookmark <id | name>
    play <selection-criteria>

    next <count>        (alias: n)
    prev <count>        (alias: p)

    repeat {{ all | one | off }}
    stop
    pause
    fasforward <duration> <time-unit>   (aliases: ff, fwd)
    rewind <duration> <time-unit>       (aliases: rw, rwd)
    jump <id | name>

Playlist and Queue Management:

    enqueue selection
    enqueue <selection-criteria>

    add selection to playlist to playlist <id | name>
    add <selection-criteria> to playlist <id | name>

    remove selection from playlist <id | name>
    remove <selection-criteria> from playlist <id | name>
    remove selection from {{ queue | selection }}
    remove <selection-criteria> from {{ queue | selection }}

    create bookmark named <name>
    create bookmark named <name> on playlist <playlist name | id> at <file name | id>
    create playlist named <name> from {{ selection | queue }}
    create playlist named <name from <selection-criteria>

    copy playlist <id | name> as <new name>
    create bookmark <id | name>
    update bookmark <id | name>

    delete playlist <id | name>
    delete bookmark <id | name>

    randomize {{ queue | playlist | selection }}

    reorder queue move <file id | name> to <position>
    reorder playlist <id | name> move <file id | name> to <position>
    reorder queue as <file id>...
    reorder playlist <id | name> as <file id>...

    clear
    clear queue
    clear selection
    clear playlist <id | name>

Library Management:

    scan
""")
